##############################################################################
#    Copyright (C) 2001  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################
"""
"""

ADD_CONTENT_PERMISSION = 'Add portal content'

#Globals
import Globals
import zLOG
from AccessControl import ClassSecurityInfo
from DateTime.DateTime import DateTime
from Acquisition import aq_parent, aq_inner, aq_base

#CMF
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.CMFCorePermissions import View
from Products.CMFCore.CMFCorePermissions import ManageProperties
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore.utils import getToolByName
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl

#MMMCore
from MasterFile import MasterFile
from General import TimeStamp
import General

factory_type_information = ({
      'id': 'Clip',
      'meta_type': 'Media Manager Clip',
      'description': 'Use clip to make new Media Clip',
      'product': 'MyMediaManager',
      'icon': 'clip_icon.png',
      'factory': 'addMediaClip',
      'filter_content_types': 0,
      'immediate_view': 'mediaclip_edit_form',
      'actions':
	      ({'name': 'View',
	        'action': 'mediaclip_view',
		'permissions': ('View',),
		'category': 'object'},
	       {'name': 'Edit',
	        'action': 'mediaclip_edit_form',
		'permissions': ('Modify portal content',),
		'category': 'object'})
       },)


class MediaClip( PortalContent
          , DefaultDublinCoreImpl
          ):
    """
        A Link
    """

    meta_type='Media Manager Clip'
    portal_type='Object'
    effective_date = expiration_date = None
    _isDiscussable = 1

    tcin = TimeStamp()
    tcout = TimeStamp()
    link  = ''
    altlink = ''
    masterfile = None
    body = ''
    smilname = ''
    rpmname = ''
    identifier = ''
    last_ftp_date = ''

    security = ClassSecurityInfo()

    def __init__( self
                , id
                , title=''
                , description=''
                ):
        DefaultDublinCoreImpl.__init__(self)
        self.id=id
        self.title=title
        self.description=description
        self.last_ftp_date = ''
    	self.masterfile_path = None
        zLOG.LOG("MediaCinema", -100, "Made MasterFile %s" % (id))

    def setMasterFilePath(self, masterfile_path=None):
        """
        Sets the master file to the one specified in masterfile.
        Param:masterfilee - a string with the ful path to the masterfile object
        """
        if masterfile_path is not None:
            self.masterfile_path = masterfile_path


    def getMasterFile(self):
        purl = getToolByName(self, 'portal_url').getPortalPath()
        mpath = self.masterfile_path
        if mpath[0] != '/':
            mpath = "%s/%s" % (purl, mpath)
        else:
            mpath = "%s%s" % (purl, mpath)
        return mpath

    def getMasterFileObject(self):
	zLOG.LOG('Clip', zLOG.INFO, 'Returning masterfile object')
        return self.restrictedTraverse(self.getMasterFile())

    security.declarePublic('getParentEvent') # obsolete
    def getParentEvent(self):
        parent = self.aq_inner.aq_parent
        return parent

    def setTimeCodes(self, tcin, tcout):
    	"""
        Sets the Time clip data.
        """
        if General.isinstance(tcin, TimeStamp):
            self.tcin = tcin
        else:
            self.tcin = TimeStamp(tcin)

        if General.isinstance(tcout, TimeStamp):
            self.tcout = tcout
        else:
            self.tcout = TimeStamp(tcout)

    def editLink(self, link, altlink):
        self.link = link
        self.altlink = altlink

    def setLastFtpDate(self, ftp_date):
        if ftp_date is not None:
            self.last_ftp_date = ftp_date

    def SearchableText(self):
        """
            text for indexing
        """
        return "%s %s" % (self.title, self.description)

    def editClip(self, body=None, identifier=None, smilname=None, rpmname=None):
        if body is not None:
            self.body = body

        if identifier is not None:
            self.identifier = identifier

        if smilname is not None:
            self.smilname = smilname

        if rpmname is not None:
            self.rpmname = rpmname

    security.declareProtected(ManageProperties, 'edit')
    def edit(self, title=None, description=None, effective_date=None, expiration_date=None):
        if title is not None:
            self.setTitle(title)

        if description is not None:
            self.setDescription(description)

        if effective_date is not None:
            self.setEffectiveDate(effective_date)

        if expiration_date is not None:
            self.setExpirationDate(expiration_date)

    security.declarePublic('Body')
    def Body(self):
        """
        Return the body text of the clip
        """
        return self.body

    security.declarePublic('syn_url')
    def syn_url(self):
        """
        Return the url to be syndicated
        """
        url = "%s?start=%s&end=%s" % (self.masterfile.lookupPlayerSource(), self.tcin, self.tcout)
        zLOG.LOG('Clip', zLOG.INFO, 'syn_url must be changed to use media tool')
        return url
        #e_obj = self.restrictedTraverse('..')
        #if e_obj.usePixregion == 'true':
        #	src = 'SmilGenPix.smil'
    	#else:
	    #	src = 'SmilGen.smil'
	    #return "%s/%s" % (self.absolute_url(), src)

    security.declarePublic('getSMIL')
    def getSMIL(self, context = None):
        """
        Returns a SMIL object
        """
        if context is None:
            return None
        #elif


def addMediaClip(self, id, title='', REQUEST=None):
	ob=MediaClip(id,title)
	self._setObject(id, ob)

Globals.InitializeClass(MediaClip)

from Products.CMFCore.register import registerPortalContent
registerPortalContent(MediaClip,
                      constructors=(addMediaClip,),
		              action='mediaclip_edit_form',
                      icon='clip_icon.png')
