##############################################################################
#    Copyright (C) 2001  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################
"""
"""

ADD_CONTENT_PERMISSION = 'Add portal content'

#Python
from types import *
import urllib, os, string, sys
import shutil

#Globals
import zLOG
import Globals
from Globals import PersistentMapping, MessageDialog
from OFS.content_types import guess_content_type
from Persistence import Persistent
from DateTime.DateTime import DateTime
from App.Common import package_home
from AccessControl import *

#CMF
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.CMFCorePermissions import View
from Products.CMFCore.CMFCorePermissions import ManageProperties
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore.WorkflowCore import WorkflowMethod
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CMFCore.utils import tuplize, getToolByName

from AccessControl import ClassSecurityInfo, getSecurityManager
from Acquisition import aq_parent, aq_inner, aq_base

import General
from General import *
from Medianet import Medianet

NO_BACKUPS = 0
BACKUP_ON_DELETE = 1
ALWAYS_BACKUP = 2
UNDO_POLICY = BACKUP_ON_DELETE
#bad =' ()[]{}~J~Z~_~N~^'
#good='_______AAAAAAaaaaaaCcEEEEeeeeeIIIIiiiiNnOOOOOOooooooSssUUUUuuuuYYyyZz'
#TRANSMAP=string.maketrans(bad, good)

factory_type_information = ({
      'id': 'Master File',
      'meta_type': 'Media Manager Master File',
      'description': 'Use master file to store link to files',
      'product': 'MyMediaManager',
      'icon': 'masterfile_icon.png',
      'factory': 'addMasterFile',
      'filter_content_types': 0,
      'immediate_view': 'masterfile_edit_form',
      'actions':
	      ({'name': 'View',
	        'action': 'masterfile_view',
		'permissions': ('View',),
		'category': 'object'},
	       {'name': 'Edit',
	        'action': 'masterfile_edit_form',
		'permissions': ('Modify portal content',)})
       },)

class MasterFileObject(Persistent):
    masterFile = None
    relativeTime = None
    content_type = None
    filesize = None
    xsize = None
    ysize = None

    security = ClassSecurityInfo()

    def _datify( self, attrib):
        if attrib == 'None':
            attrib = TimeStamp()
        elif not General.isinstance( attrib, TimeStamp ):
	        if attrib is not None:
	            attrib = TimeStamp( attrib )
        return attrib

    def __init__(self, masterFile, logic, time):
        self.masterFile = masterFile
        self.logic = logic
        self.relativeTime = self._datify(time)
        self.content_type = ''
        self.filesize = 0
        self.xsize = 240
        self.ysize = 180

    security.declarePublic('getURL')
    def getURL(self):
        logic = self.masterFile.medianet[self.logic]
        return "%s://%s:%s%s%s%s" % (logic.protocol, logic.host, logic.port, logic.path, self.masterFile.filename, logic.ekstension)

    security.declarePublic('getRelated')
    def getRelated(self):
        logic = self.masterFile.medianet[self.logic]
        return "%s://%s%s%s%s" % (logic.protocol, logic.ppv_serverid, logic.path, self.masterFile.filename, logic.ekstension)

    security.declarePublic('getTime')
    def getTime(self):
        return self.relativeTime

    security.declareProtected(ManageProperties, 'setTime')
    def setTime(self, time):
        self.relativeTime = self._datify(time)

    security.declareProtected(ManageProperties, 'setContentType')
    def setContentType(self, content_type):
        self.content_type = content_type

    security.declarePublic('getContentType')
    def getContentType(self):
        return self.content_type

    security.declareProtected(ManageProperties, 'setFileSize')
    def setFileSize(self, filesize):
        self.filesize = filesize

    security.declarePublic('getFileSize')
    def getFileSize(self):
	    return self.filesize

    security.declareProtected(ManageProperties, 'setXSize')
    def setXSize(self, xsize):
        self.xsize = xsize

    security.declareProtected(ManageProperties, 'setYSize')
    def setYSize(self, ysize):
        self.ysize = ysize

    security.declarePublic('getXSize')
    def getXSize(self):
        return self.xsize

    security.declarePublic('getYSize')
    def getYSize(self):
        return self.ysize


class MasterFile( PortalContent
                , DefaultDublinCoreImpl
                , Playable ):
    """
        A Link
    """

    meta_type='Media Manager Master File'
    portal_type='Object'
    effective_date = expiration_date = None
    _isDiscussable = 1
    tracks = []
    medianet_path = None
    security = ClassSecurityInfo()
    encoder_start = ''
    state = 'normal'

    def __init__( self
                , id
                , title=''
                , description=''
		        , xsixe=''
		        , ysize=''
	        	, filename=''
        		, content_type=''
                ):
        DefaultDublinCoreImpl.__init__(self)
        self.id = id
        self.title = title
        self.logics = {}
        self.description = description
        self.xsize = 0
        self.ysize = 0
        self.filename = ''
        self.content_type = ''
        self.encoder_start = ''

    security.declarePrivate('_initMasterFile')
    def _initMasterFile(self):
        """
        assigns medianet
        assigns default live and on demand servers
        uploads wait file
        """
        catalog = getToolByName(self, 'portal_catalog')
        medianet = getToolByName(self, 'portal_mediamanager').getMedianetFor(self)
        mediamanager = getToolByName(self, 'portal_mediamanager')
        self.setMedianetPath(medianet.absolute_url(relative=1))
        live_logic = medianet.getDefaultLive()
        ondemand_logic = medianet.getDefaultOndemand()
        self.clearLogics()
    	self.assignLogic(live_logic, '00:00.000')
        self.assignLogic(ondemand_logic, '00:00.000')

    	path = package_home(globals())
        realfile = 'mediafiles/wait.rm'
    	realpath = os.path.join(path, realfile)
        wmfile = 'mediafiles/wait.wmv'
    	wmpath = os.path.join(path, wmfile)
        zLOG.LOG('Masterfile', zLOG.INFO, 'Windows media file home: %s' % wmpath)
        zLOG.LOG('Masterfile', zLOG.INFO, 'Real media file home: %s' % realpath)

        username = str(getSecurityManager().getUser())
    	newid = mediamanager.getUniqueID()
        fn = '%s' % newid
    	wfilename = 'wait'
        for logicpath in self.listLogics():
            logic = self.restrictedTraverse(logicpath)
            zLOG.LOG('Master File Init', zLOG.INFO, 'Logic value: %s, type: %s' % (logic.id, logic.servertype))
            ext = logic.extension
            remote_filename = fn + ext
    	    if logic.servertype == 'real':
                filepath = realpath
            elif logic.servertype == 'windows6':
                filepath = wmpath
            elif logic.servertype == 'windows7':
                filepath = wmpath
            else:
                raise 'Error when initialize masterfile', 'Logic\'s servertype not defined'

            try:
                file = open(filepath)
                zLOG.LOG('Masterfile init', zLOG.INFO, 'Trying to upload file: %s\nLocal filepath: %s' % (remote_filename, filepath))
                if (hasattr(logic, 'ftp_dir') and logic.ftp_dir) == '':
                    resp = self.ftp_out(file=file, host=logic.host, login=logic.ftp_username, password=logic.ftp_password, remote_name=remote_filename)
                else:
                    resp = self.ftp_out(file=file, host=logic.host, login=logic.ftp_username, password=logic.ftp_password, remote_directory=logic.ftp_dir, remote_name=remote_filename)
                zLOG.LOG('Masterfile', zLOG.INFO, resp)
            except:
                zLOG.LOG('Masterfile', zLOG.INFO, 'Added masterfile - upload of wait files failed')
                pass # just to be able to make some tests as logic doesn't have these attributes (thomas)

    	copytime = str(DateTime().strftime('%Y'))
        copytext = 'Copyright %s %s' % (copytime, username)
    	self.setRights(copytext)
        self.filename = fn

    def setMedianetPath(self, medianetpath):
        """
        set the path to the medianet. currently we only use one medianet
        XXX is this obsolete
        """
        self.medianet_path = medianetpath
        self._p_changed = 1

    def getMedianetObject(self):
        """
        returns the medianet object.
        XXX maybe this should call portal_mediamanager.getMedianetFor(self)
        """
        medianet = self.restrictedTraverse(self.medianet_path)
        return medianet

    security.declarePublic('DumpLOG')
    def DumpLOG(self, REQUEST):
    	zLOG.LOG("MasterFile", 0, "%s" % (str(REQUEST)))

    def listLogics( self ):
        """
    	  List All offsets available
        """
    	return self.logics.keys()

    def listLogicObjects( self ):
        """
        List All offsets available
        if we get an exception other than Unauthorized we delete it from the list.
        """
        keys = self.logics.keys()
        logics = []
        for logicpath in keys:
            try:
                logic = self.restrictedTraverse(logicpath)
                logics.append(logic)
            except Unauthorized: # we dont wanna delete it if the exception is because we're not allowed to see it
                pass
            except:
                del self.logics[logicpath]
        return logics

    security.declareProtected(ManageProperties, 'setFilename')
    def setFilename(self, filename):
        """
        set the filename (without extension?)
        """
        self.filename = filename

    security.declareProtected(View, 'checkLogic')
    def checkLogic( self, id ):
        """
    	   Checks if the specified logic applies to this masterfile
        """
    	return self.logics.has_key(id)

    security.declarePublic('getLogic')
    def getLogic( self, id ):
        """
        returns the relative path to the logic with index 'id'
        XXX non-intuitive
        """
        zLOG.LOG('Master file class', zLOG.INFO, 'Requesting id: %s' % id)
        return self.logics[id]

    security.declareProtected(View, 'getLogicObject')
    def getLogicObject(self, id):
        """
        returns the object of the logic with relative path 'id'
        XXX non-intuitive
        """
        zLOG.LOG('Medianet', zLOG.INFO, 'Logic id requesting: %s' % id)
        #return self.medianet[id]
        try:
            return self.restrictedTraverse(id)
        except:
            return None

    security.declareProtected(ManageProperties, 'deleteLogic')
    def deleteLogic(self, id):
        """
        delete the logic with index (sic!) id
        XXX non-intuitive
        """
        del self.logics[id]
        self._p_changed = 1

    security.declareProtected(ManageProperties, 'assignLogic')
    def assignLogic( self, id, offset=None, upload_waitfile=1 ):
        """
    	   Assigns a Logic and an Offset to the Masterfile.
            Uploads a waitfile if upload_waitfile equals 1.
        """
    	#raise "DEBUG", "Assign %s to %s" % (id, offset)
        self.logics[id] = MasterFileObject(masterFile = self, logic=id, time=offset)

        if upload_waitfile:
            zLOG.LOG('Masterfile', zLOG.INFO, 'Uploading wait file')
            path = package_home(globals())
            realfile = 'mediafiles/wait.rm'
            realpath = os.path.join(path, realfile)
            wmfile = 'mediafiles/wait.wmv'
            wmpath = os.path.join(path, wmfile)
            zLOG.LOG('Masterfile', zLOG.INFO, 'Package home: %s' % wmpath)
            wfilename = 'wait'
            logic = self.restrictedTraverse(id)
            ext = logic.extension
            remote_filename = self.filename + ext
            if logic.servertype == 'real':
                filepath = realpath
            elif logic.servertype == 'windows6':
                filepath = wmpath
            elif logic.servertype == 'windows7':
                filepath = wmpath
            else:
                raise 'Error when uploading wait file', 'Logic\'s servertype not defined'
            try:
                file = open(filepath)
                if (hasattr(logic, 'ftp_dir') and logic.ftp_dir) == '':
                    resp = self.ftp_out(file=file, host=logic.host, login=logic.ftp_username, password=logic.ftp_password, remote_name=remote_filename)
                else:
                    resp = self.ftp_out(file=file, host=logic.host, login=logic.ftp_username, password=logic.ftp_password, remote_directory=logic.ftp_dir, remote_name=remote_filename)
            except:
                pass
            logicobj = self.getLogic(id)
            content_type = logicobj.getContentType()
            content_type = self._get_content_type(file, file.read(100), self.id, content_type or logicobj.getContentType())
            logicobj.setContentType(content_type)
        else:
            zLOG.LOG('Masterfile', zLOG.INFO, 'Not uploading wait files')
        self._p_changed = 1

    security.declareProtected(ManageProperties, 'clearLogics')
    def clearLogics( self ):
        """
        empties the server logic list
        """
        #raise "DEBUG", "Clear %s" % (self.logics.values())
        self.logics.clear()
        self._p_changed = 1

    def setEncoderStart(self, enc_start=None):
        """
        """
        if enc_start is not None:
            self.encoder_start = enc_start

    def SearchableText(self):
        """
            text for indexing
        """
        return "%s %s" % (self.title, self.description)

    def Related(self):
        """
        returns a list of urls related to this object.
        For a masterfile this is all urls to the server files.
        """
        relate = []
        for logic in self.logics.values():
            relate.append(logic.getRelated())
        return tuplize('related', relate)

    def editMasterFile(self, title=None, description=None, encoder_start=None, filename=None):
        if title is not None:
            self.title=title

        if description is not None:
            self.description=description

        if encoder_start is not None:
            self.encoder_start = encoder_start

        if filename is not None:
            self.filename = filename

    def cb_isMoveable(self):
        return 0

    def cb_isCopyable(self):
        """
        You can copy this object
        """
        return 1

    def editFile(self, file='', xsize=None, ysize=None, time=None, logicid=None):
        zLOG.LOG('Masterfile.editFile', zLOG.INFO, "File: %s - xsize: %s - ysize: %s - time: %s logicid: %s" %
                    (file, xsize, ysize, time, logicid))
        res = ''
        if logicid is not None:
            if self.checkLogic(logicid):
                logicobj = self.getLogic(logicid)
                logicobj.setXSize(xsize)
                logicobj.setYSize(ysize)
                logicobj.setTime(time)
                if file.filename != '':
                    res = self.manage_file_upload(file=file, logicid=logicid)
                self.setFormat(self.content_type)
            else:
                raise 'Logic error', "Server id %s wasn't found" % logicid
        return res

    def manage_file_upload(self, file='', content_type='', logicid=None, REQUEST=None):
    	if type(file) is not type(''):
	       reposit=self._reposit_path()
        if self.filename != '':
            new_writefn = self._get_writepathfn(file)
            new_fn = self.filename
        else:
            fileError = 'Masterfileobject has no filename'
            if self.filename == '' or file.filename == '':
                raise fileError, 'Filename attrib: %s' % (self.filename)
            else:
                raise fileError, 'Files filename attrib: %s' % (file.filename)
        resp = ('','')
        if logicid is not None:
            if self.checkLogic(logicid):
                logic = self.restrictedTraverse(logicid)
            if logic.ftp_username != '' and logic.ftp_password != '':
                filename = self.filename + logic.extension
                if logic.ftp_dir == '':
                    resp = self.ftp_out(file=file, host=logic.host, login=logic.ftp_username, password=logic.ftp_password, remote_name=filename)
                else:
                    resp = self.ftp_out(file=file, host=logic.host, login=logic.ftp_username, password=logic.ftp_password, remote_directory=logic.ftp_dir, remote_name=filename)
        else:
            raise 'Logic error', 'Logic id is not defined'
        logicobj = self.getLogic(logicid)
        content_type = logicobj.getContentType()
        content_type = self._get_content_type(file, file.read(100),self.id, content_type or logicobj.getContentType())
        logicobj.setContentType(content_type)
        self.filename = new_fn
        return resp[1]

    def PUT(self, REQUEST, RESPONSE):
        """
        Handle HTTP PUT requests
        """
        reposit = self._reposit_path()

        # in this version we propably have to do it another way (thomas)
        for logicpath in self.listLogics():
            tmp_logic = self.restrictedTraverse(logicpath)
            if tmp_logic.meta_type == 'Media Manager Server Logic':
                logic = tmp_logic
                zLOG.LOG('Masterfile.PUT', zLOG.INFO, 'Logic path: %s' % logic.absolute_url())
        self.dav__init(REQUEST, RESPONSE)
        self.state = 'uploading'
        content_type = REQUEST.get_header('content-type', None)
        content_length = REQUEST.get_header('Content-Length', None)
        #fn = string.split(REQUEST['HTTP_CONTENT_DISPOSITION'],'=') # if we ever need the original filename (thomas)
        instream = REQUEST['BODYFILE']

        if not self.filename:
            self.state = 'error'
            RESPONSE.setStatus(401)
            return RESPONSE

        if logic: # we have no garantee that there is a logic by now
            zLOG.LOG('Masterfile.PUT', zLOG.INFO, 'We''ve got a logic')
            try:
                zLOG.LOG('Masterfile.PUT', zLOG.INFO, 'Info: host: %s - user: %s - pwd: %s - remote_fn: %s - remote_path: %s' % (logic.host, logic.ftp_username, logic.ftp_password, self.filename, logic.ftp_dir))
                filename = '%s%s' % (self.filename, logic.getFileExtension())
                if logic.ftp_dir == '':
                    resp = self.ftp_out(file=instream, host=logic.host, login=logic.ftp_username, password=logic.ftp_password, remote_name=filename)
                else:
                    resp = self.ftp_out(file=instream, host=logic.host, login=logic.ftp_username, password=logic.ftp_password, remote_directory=logic.ftp_dir, remote_name=filename)
            except:
                zLOG.LOG('Masterfile.PUT', zLOG.INFO, 'Sys: %s' % str(sys.exc_value))
            zLOG.LOG('Masterfile.PUT', zLOG.INFO, 'Response from ftp upload: %d - %s' % (resp[0], resp[1]))
            if resp[0] < 200 or resp[0] >= 300:
                zLOG.LOG('Masterfile.PUT', zLOG.INFO, 'WE RULE TO BAD!!!!!')
                self.state = 'archived'
                RESPONSE.setStatus(resp[0])
                return RESPONSE
        else:
            zLOG.LOG('Masterfile.PUT', zLOG.INFO, 'We have no logic whatsoever')
            self.status = 'error'
            RESPONSE.setStatus(401)
            return RESPONSE
        self.content_type = str(content_type)
        #self.content_type = self._get_content_type(instream.read(100), self.id, content_type or self.content_type)
        if new_fn:
            self.filename = new_fn
        RESPONSE.setStatus(204)
        return RESPONSE

    def _copy_data(self, instream, outstream):
        if type(instream) is StringType:
            outstream.write(instream)
        else:
            try:
                instream.seek(0,2)
                size=instream.tell()
                instream.seek(0)
                blocksize=2<<16
                pos=0
                while pos<size:
                    outstream.write(instream.read(blcksize))
                    pos=pos+blocksize
                instream.seek(0)
            except:
                outstream.write(instream.read())
	
    def _get_writeufn(self):
        """Creates a unique filename"""
        udir = str(getSecurityManager().getUser())
        reposit=self._reposit_path()
        ureposit=os.path.join(reposit, udir)
        time=int(DateTime().strftime('%m%d%H%M%S'))
        fn=os.path.join(ureposit, "%s_%d_%s"%(udir,time,self.id))
        while (os.isfile(fn) or os.isfile(fn+'.preview') or os.isfile(fn+'.undo') or os.isfile(fn+'.preview.undo')):
            time=time+1
        fn=os.path.join(reposit,"%s_%d_%s"%(udir,time,self.id))
        new_fn=fn+'.rm'
        return new_fn

    def _get_ufn(self):
        """Creates a unique filename"""
        udir = str(getSecurityManager().getUser())
        time=int(DateTime().strftime('%m%d%H%M%S'))
        fn="%s_%d_%s"%(udir,time,self.id)
        new_fn = os.path.join(udir, fn)
        correct_fn=new_fn+'.rm'
        return correct_fn

    def _get_writepathfn(self, file):
        """Create a filename with the correct path"""
        udir = str(getSecurityManager().getUser())
        reposit=self._reposit_path()
        ureposit=os.path.join(reposit, udir)
        if self.filename=='':
            new_fn=os.path.join(ureposit, file.filename)
        else:
            new_fn=os.path.join(reposit, self.filename)
        return new_fn

    def _get_filename(self, file):
        """Get filename + dir where file is putted"""
        udir = str(getSecurityManager().getUser())
        if file.filename == '':
            time=int(DateTime().strftime('%m%d%H%M%S'))
            fn="%s_%d_%s"%(udir,time,self.id)
            new_fn=os.path.join(udir, fn)
        else:
            new_fn = os.path.join(udir, file.filename)
        return new_fn

    def _reposit_path(self):
        path=INSTANCE_HOME
        return path

    def _enc_reposit_path(self):
        path=INSTANCE_HOME
        for item in self._enc_repository:
            path=os.path.join(path, item)
        return path

    def _get_content_type(self, file, body, id, content_type=None):
        """Determine the mime-type"""
        headers=getattr(file, 'headers', None)
        if headers and headers.has_key('content_type'):
            content_type=headers['content_type']
        else:
            if type(body) is not type(''): body=body.data
            content_type, enc=guess_content_type(getattr(file, 'filename', id), body, content_type)
        return content_type

def addMasterFile(self, id, title='', REQUEST=None):
	ob=MasterFile(id,title)
	self._setObject(id, ob)
	file = self._getOb(id)
	file._initMasterFile()

Globals.InitializeClass(MasterFile)

from Products.CMFCore.register import registerPortalContent
registerPortalContent(MasterFile,
                    constructors=(addMasterFile,),
		            action='masterfile_edit_form',
                    icon='masterfile_icon.png')
