##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

"""Basic portal media manager access tool.
$Id: MediaManagerTool.py,v 1.18 2002/04/11 09:55:46 gitte Exp $
"""
__version__='$Revision: 1.18 $'

import os, zLOG, random
from Globals import InitializeClass, DTMLFile, package_home
from AccessControl import ClassSecurityInfo
from OFS.SimpleItem import SimpleItem

from Products.CMFCore.utils import UniqueObject, getToolByName
from Products.CMFCore import CMFCorePermissions
from General import _dtmldir
from MediaLiveEvent import MediaLiveEvent
from DateTime import DateTime

try:
    from Products.MMMSmilEditor.SmilObject import SmilObject
    smil_installed = 1
except ImportError:
    smil_installed = 0

try:
    from Products.MMMRealPixEditor.MediaSlideShow import MediaSlideShow
    slide_installed = 1
except ImportError:
    slide_installed = 0


class MediaManagerException( Exception ):
    pass

class ServerUrl(SimpleItem):
    def __init__(self, logic, userprefs, url):
        self.logic = logic
        self.userprefs = userprefs
        self.url = url

    def __cmp__(self, other):
        if other is not None:
            return cmp(self.isPreferred(), other.isPreferred())
        else:
            return 1

    def isPreferred(self):
        isPref = 0
        logicpath = self.logic.absolute_url(relative=1)
        zLOG.LOG('ServerUrl', zLOG.INFO, 'Logicpath: %s' % logicpath)
        if self.logic.servertype == self.userprefs['player']:
            isPref = 1
        if not isPref:
            if logicpath == self.userprefs['server']:
                isPref = 1
        else:
            if logicpath == self.userprefs['server']:
                isPref = 2
        return isPref

class MediaManagerTool( UniqueObject, SimpleItem ):

    id = 'portal_mediamanager'
    meta_type = 'Media Manager Tool'

    _allowed_meta_types = [
                             'Media Manager Live Event'
                            ,'Media Manager Show'
                            ,'Media Manager Clip'
                            ,'Media Sub Event'
                            ,'Media Manager Smil Data'
                            ,'Media Manager Slide Show'
                            ,'Media Manager Master File'
                            ]

    security = ClassSecurityInfo()

    manage_options = ( { 'label' : 'Overview', 'action' : 'manage_overview' }
                     ,
                     ) + SimpleItem.manage_options

    #
    #   ZMI methods
    #
    security.declareProtected( CMFCorePermissions.ManagePortal
                             , 'manage_overview' )
    manage_overview = DTMLFile( 'explainMediaManagerTool', _dtmldir )

    #
    #   'portal_mediamanager' interface methods
    #

    security.declarePublic( 'getMedianetFor' )
    def getMedianetFor(self, content):
        """
        Returns a Medianet object or None.
        Currently it takes only <portal_url>/medianet.
        Currently content isn't used but that may change.
        """
        has_net = 0
        medianet = None
        portal = getToolByName(self, 'portal_url').getPortalObject()
        if 'medianet' in portal.objectIds():
            zLOG.LOG('Media Tool', zLOG.INFO, 'getMedianetFor(%s) - found one in %s' % (content.getId(), portal.getId()))
            medianet = portal.restrictedTraverse('medianet')
            if medianet.meta_type == 'Media Manager Medianet':
                zLOG.LOG('Media Tool', zLOG.INFO, 'getMedianetFor(%s) - and it has correct meta_type' % content.getId())
                has_net = 1
            else:
                zLOG.LOG('Media Tool', zLOG.INFO, 'getMedianetFor(%s) - wrong meta_type: %s' % medianet.meta_type)
        else:
            zLOG.LOG('Media Tool', zLOG.INFO, 'getMedianetFor(%s) - no medianet found in %s' % (content.getId(), portal.objectIds()))
        if not has_net:
            raise "No Medianet - you have to create a 'Media Manager Medianet' called 'medianet' at the root of the portal"
        return medianet

    security.declarePublic('getUniqueID')
    def getUniqueID(self, prefix=None):
        if prefix is not None:
            newid = prefix
        else:
            newid = ''
        newid = newid + str(DateTime().millis()) + str(random.randint(0, 1000000))
        return newid

    security.declarePublic( 'getPlayUrlsFor' )
    def getPlayUrlsFor(self, content, direct = 1):
        """
            Return a list of dictionaries containing urls to be used in the player.

            Arguments

                o content - the object for which we want play urls.

                o direct - whether we want to access the media directly
                  (like an rtsp:// protocol url) or indirectly (like a SMIL file)

            The dictionaries have the following keys:

                o url - The url to be used

                o preferred - A boolean value saying if the url points to a media file
                  in the preferred format. This is based on settings in the medianet
                  and the users preferences.

                o format - a string containing the media format the url points to.

            On error the method returns None
        """
        medianet = self.getMedianetFor(content)
        if not medianet: return None
        urls = []
        userprefs = {}
        if not self.isMediaManagerType(content):
            # we can't play a document can we?
            return None
        memtool = getToolByName(self, 'portal_membership')
        if memtool.isAnonymousUser():
            try:
                pref_player = medianet.getPrefPlayer() #(will implement this soon !!!)
                pref_liveserver = medianet.getDefaultLive()
                pref_archiveserver = medianet.getDefaultOndemand()
            except:
                pref_player = ''
                pref_liveserver = ''
                pref_archiveserver = ''
        else:
            member = memtool.getAuthenticatedMember()
            try:
                pref_player = member.pref_player
                pref_liveserver = member.pref_liveserver
                pref_archiveserver = member.pref_archiveserver
            except:
                pref_player = ''
                try:
                    pref_liveserver = medianet.getDefaultLive()
                    pref_archiveserver = medianet.getDefaultOndemand()
                except:
                    pref_liveserver = ''
                    pref_archiveserver = ''
        userprefs['player'] = pref_player

        temp_e = self.getEventFor(content)
        if self.isLive(temp_e):
            logic_type = 'Media Manager Live Server Logic'
            userprefs['server'] = pref_liveserver
        else:
            logic_type = 'Media Manager Server Logic'
            userprefs['server'] = pref_archiveserver

        if content.meta_type == 'Media Manager Live Event' or content.meta_type == 'Media Manager Show':
            # here we can return an object per media type.
            # If 'direct' is true:
            # if it is Real Media we return a SMIL file, if it is Windows Media we return a SAMI (?) file
            # If 'direct' is false:
            # if it is Real Media we return an rtsp url, if it is Windows Media we return an XXX url ?
            # this routine should change but we need a new way of listing server types e.g.
            #   [
            #        {
            #            'id':'real'
            #           ,'title':'Real Media'
            #           ,'sequence_generator':'SMIL.smil'
            #        }
            #       ,{
            #            'id':'windows'
            #           ,'title':'Windows Media'
            #           ,'sequence_generator':'SAMI.sam'
            #        }
            #   ]
            if content.meta_type == 'Media Manager Live Event':
                e = content
            else:
                e = self.getEventFor(content)
            if e:
                mfiles = e.listFileObjects()
            else:
                raise MediaManagerException, "Couldn't find event for %s" % content.absolute_url(relative=1)
            for file in mfiles:
                for logicpath in file.listLogics():
                    logic = self.restrictedTraverse(logicpath)
                    if logic.meta_type == logic_type:
                        if not direct:
                            if logic.servertype == 'real':
                                url = '%s/SMIL.smil' % content.absolute_url()
                                urls.append(ServerUrl(logic=logic, userprefs=userprefs, url=url))
                            elif logic.servertype == 'windows6':
                                url = '%s/AsxGen.asx' % content.absolute_url()
                                urls.append(ServerUrl(logic=logic, userprefs=userprefs, url=url))
                        else:
                            if self.isLive(e) and logic.servertype == 'windows6':
                                url = "%s%s%s" % (logic.getPlayerUrl(), file.filename, logic.getFileExtension())
                                zLOG.LOG('Media Tool', zLOG.INFO, 'Player url: %s' % url)
                                urls.append(ServerUrl(logic=logic, userprefs=userprefs, url=url))
                            else:
                                url = "%s%s%s" % (logic.getPlayerUrl(), file.filename, logic.getFileExtension())
                                urls.append(ServerUrl(logic=logic, userprefs=userprefs, url=url))

        elif content.meta_type == 'Media Manager Smil Data':
            # SMIL data can also be played though they are kinda static.
            # We ignore the 'direct' argument.
            format = 'Real Media'
            if format in medianet.getServerTypes():
                urls.append({
                             'url': '%s/SMIL.smil' % content.absolute_url()
                            ,'preferred': 1
                            ,'format': format})
        elif content.meta_type == 'Media Manager Clip':
            mobject = content.getMasterFileObject()
            for logicpath in mobject.listLogics():
                logic = self.restrictedTraverse(logicpath)
                if logic.meta_type == logic_type:
                    if not direct:
                        if logic.servertype == 'real':
                            url = '%s/SMIL.smil' % content.absolute_url()
                            urls.append(ServerUrl(logic=logic, userprefs=userprefs, url=url))
                        elif logic.servertype == 'windows6':
                            url = '%s/AsxGen.asx' % content.absolute_url()
                            urls.append(ServerUrl(logic=logic, userprefs=userprefs, url=url))
                    else:
                        if self.isLive(temp_e) and logic.servertype == 'windows6':
                            url = logic.getPlayerUrl()
                            urls.append(ServerUrl(logig=logic, userprefs=userprefs, url=url))
                        else:
                            url = "%s%s%s" % (logic.getPlayerUrl(), mobject.filename, logic.getFileExtension())
                            urls.append(ServerUrl(logic=logic, userprefs=userprefs, url=url))
        elif content.meta_type == 'Media Manager Sub Event':
            pass
        else:
            pass

        urls.sort()
        return urls

    security.declarePublic('getEditorUrls')
    def getEditorUrls(self, masterfile):

        if not masterfile.meta_type == 'Media Manager Master File':
            raise 'Object Error', 'Meta type is: %s' % masterfile.meta_type
        medianet = self.getMedianetFor(masterfile)
        if not medianet: return None
        urls = []
        userprefs = {}
        if not self.isMediaManagerType(masterfile):
            # we can't play a document can we?
            return None
        memtool = getToolByName(self, 'portal_membership')
        if memtool.isAnonymousUser():
            try:
                pref_player = medianet.getPrefPlayer() #(will implement this soon !!!)
                pref_server = medianet.getDefaultOndemand()
            except:
                pref_player = ''
                pref_server = ''
        else:
            member = memtool.getAuthenticatedMember()
            try:
                pref_player = member.pref_player
                pref_server = member.pref_archiveserver
            except:
                pref_player = ''
                try:
                    pref_server = medianet.getDefaultOndemand()
                except:
                    pref_server = ''
        userprefs['player'] = pref_player
        userprefs['server'] = pref_server
        zLOG.LOG('Media Manager Tool', zLOG.INFO, 'Meta type: %s' % masterfile.meta_type)
        logicpaths = masterfile.listLogics()

        urls = []
        for logicpath in logicpaths:
            logic = self.restrictedTraverse(logicpath)
            if logic.meta_type == 'Media Manager Server Logic':
                url = '%s%s%s' % (logic.getPlayerUrl(), masterfile.filename, logic.getFileExtension())
                urls.append(ServerUrl(logic=logic, userprefs=userprefs, url=url))

        urls.sort()
        zLOG.LOG('Media Manager Tool', zLOG.INFO, 'Editor urls: %s' % str(urls))
        return urls
        
    security.declarePublic('getWaitFilesFor')
    def getWaitFilesFor(self, event=None):
        if event is not None and event.meta_type == 'Media Manager Live Event':
            userprefs = self.getUserPrefs(content=event)
            urls = []
            mfiles = event.listFileObjects()
            for file in mfiles: 
                for logicpath in file.listLogics():
                    logic = self.restrictedTraverse(logicpath)
                    if logic.meta_type == 'Media Manager Live Server Logic':
                        url = "%s%s%s" % (logic.getPlayerUrl(), file.filename, logic.getFileExtension())
                        zLOG.LOG('Media Tool', zLOG.INFO, 'Player url: %s' % url)
                        urls.append(ServerUrl(logic=logic, userprefs=userprefs, url=url))
        else:
            raise 'AttributeError', 'Object is not Live Event type'

        urls.sort()
        urlset = urls[0]
        zLOG.LOG('Medie tool', zLOG.INFO, 'Urlset: %s' % str(urlset.__dict__))
        url = urlset.url
        zLOG.LOG('Media Tool', zLOG.INFO, 'Returned waitfile: %s' % url)
        return urlset

    security.declarePublic('getUserPrefs')
    def getUserPrefs(self, content):
        medianet = self.getMedianetFor(content)
        if not medianet: return None
        userprefs = {}
        memtool = getToolByName(self, 'portal_membership')
        if memtool.isAnonymousUser():
            try:
                pref_player = medianet.getPrefPlayer() #(will implement this soon !!!)
                pref_liveserver = medianet.getDefaultLive()
                pref_archiveserver = medianet.getDefaultOndemand()
            except:
                pref_player = ''
                pref_liveserver = ''
                pref_archiveserver = ''
        else:
            member = memtool.getAuthenticatedMember()
            try:
                pref_player = member.pref_player
                pref_liveserver = member.pref_liveserver
                pref_archiveserver = member.pref_archiveserver
            except:
                pref_player = ''
                try:
                    pref_liveserver = medianet.getDefaultLive()
                    pref_archiveserver = medianet.getDefaultOndemand()
                except:
                    pref_liveserver = ''
                    pref_archiveserver = ''
        userprefs['player'] = pref_player

        temp_e = self.getEventFor(content)
        if self.isLive(temp_e):
            logic_type = 'Media Manager Live Server Logic'
            userprefs['server'] = pref_liveserver
        else:
            logic_type = 'Media Manager Server Logic'
            userprefs['server'] = pref_archiveserver

        return userprefs

    security.declarePublic( 'isLive' )
    def isLive(self, content):
        """
        Find out whether the event is being broadcasted live or is archived for ondemand broadcast
        """
        if not self.isMediaManagerType(content):
            raise MediaManagerException, "%s isn't a MediaManager object" % content.absolute_url(relative=1)
        if content.meta_type == 'Media Manager Live Event':
           e = content
        else:
           e = self.getEventFor(content)
        if not e:
            raise MediaManagerException, "Couldn't find event for %s" % content.absolute_url(relative=1)

        now = DateTime().millis()
        start = e.start_date.millis()
        end = e.end_date.millis() + 900000 # 15 min. after

        return (now < start or now < end)
        
    security.declarePublic('setEncoderPassword')
    def setEncoderPassword(self, password):
        memtool = getToolByName(self, 'portal_membership')
        home = memtool.getHomeFolder()
        home.encoder_password = password
        
    security.declarePublic('getEncoderPassword')
    def getEncoderPassword(self):
        memtool = getToolByName(self, 'portal_membership')
        home = memtool.getHomeFolder()
        if hasattr(home, 'encoder_password'):
            return home.encoder_password
        else:
            return ''

    security.declarePublic( 'isSmilInstalled' )
    def isSmilInstalled(self):
        """
            returns a boolean value indicating whether smil is installed or not
        """
        return smil_installed

    security.declarePublic( 'isSlideShowInstalled' )
    def isSlideShowInstalled(self):
        """
            returns a boolean value indicating whether MMMRealPixEditor is installed or not
        """
        return slide_installed

    security.declarePublic( 'getSmilObjectFor' )
    def getSmilObjectFor(self, content):
        """
            Return the SMIL object for content, creating it if need be (if the smil object is installed).
        """
        if not self.isMediaManagerType(content):
            raise "%s cannot handle objects of type: %s" % (self.id, content.Type())
        if content.meta_type == "Media Manager Smil Data":
            return content
        event = self.getEventFor(content)
        if event is None:
            raise "Couldn't find event for %s" % content.getId()

        smil = event.getSmilObject()
        if smil is None:
            medianet = self.getMedianetFor(content)
            if smil_installed:
                try:
                    smil = medianet.restrictedTraverse('default_smil')
                except:
                    raise "Couldn't find default SMIL object"
        return smil

    security.declarePublic( 'getSlideshowObjectFor' )
    def getSlideshowObjectFor(self, content):
        """
            Return the slide show object for content).
        """
        if not self.isMediaManagerType(content):
            raise "%s cannot handle objects of type: %s" % (self.id, content.Type())
        if content.meta_type == "Media Manager Slide Show":
            return content
        event = self.getEventFor(content)
        if event is None:
            raise "Couldn't find event for %s" % content.getId()

        return event.getSlideshowObject()

    security.declarePublic( 'getEventFor' )
    def getEventFor(self, content, path = None):
        """
            Return the event object for content, can be itself.
            The returned event is not necessarily the event in
            which the object is contained but the event in
            which the object is shown.
            The path is the url of the topmost frame ('/Members/foobar/MyClip') - if it is 'None'
            the returned value is either the event in which the
            object is contained or 'None'.
        """
        zLOG.LOG('portal_mediamanager', zLOG.INFO, 'getEventFor: %s' % (content.absolute_url(relative=1)))
        if content.meta_type == 'Media Manager Live Event':
            zLOG.LOG('portal_mediamanager', zLOG.INFO, 'getEventFor - Returning self as it is an Event')
            return content
        event = None
        tmp = 1
        if path is None:
            # traverse upwards until we find an event object
            tmp = content
            while event is None and tmp is not None and not getattr(tmp, '_isPortalRoot', 0):
                if hasattr(tmp, 'aq_parent'):
                    tmp = getattr(tmp, 'aq_parent')
                else:
                    tmp = tmp.restrictedTraverse('..') #aq_parent(aq_inner(tmp))
                zLOG.LOG('portal_mediamanager', zLOG.INFO, 'getEventFor: Finding Event %s' % tmp.absolute_url())
                if tmp.meta_type == "Media Manager Live Event":
                    event = tmp
                    break
        else:
            tmp = self.restrictedTraverse(path, None)
            while event is None and tmp is not None and not getattr(tmp, '_isPortalRoot', 0):
                tmp = aq_parent(aq_inner(ob))
                zLOG.LOG('portal_mediamanager', zLOG.INFO, 'getEventFor: %s %s' % (tmp.absolute_url(), content.absolute_url()))
                if tmp.meta_type == "Media Manager Live Event":
                    event = tmp
            zLOG.LOG('portal_mediamanager', zLOG.INFO, 'getEventFor: %s %s' % (event.getId(), event.meta_type))
        if (event is not None and event.meta_type == "Media Manager Live Event"):
            return event
        else:
            return None

    security.declarePublic( 'isMediaManagerType' )
    def isMediaManagerType( self, content ):
        '''
            Returns a boolean indicating whether content...
        '''
        resp = content.meta_type in self._allowed_meta_types
        zLOG.LOG('portal_mediamanager', zLOG.INFO, 'isMediaManagerType: %d' % (resp))
        return resp

    #
    #   ActionProvider interface
    #
    security.declarePublic( 'listActions' )
    def listActions(self, info):
        # Return actions for reply and show replies
        content = info.content
        if info is None or not self.isMediaManagerType(info):
            return None

        actions = (
            {'name': 'SMIL',
             'url': info.absolute_url() + '/SMIL',
             'permissions': ['View'],
             'category': 'object'
             },
            )

        zLOG.LOG('portal_mediamanager', zLOG.INFO, 'listActions: %s' % (str(actions)))
        return actions

    #
    #   Utility methods
    #
    security.declarePrivate( '_createSmilObjectFor' )
    def _createSmilObjectFor( self, content ):
        """
            Create a SMIL object in the event that content is in context off.
        """
        if not smil_installed:
            return None
        event = self.getEventFor(content)
        if not event:
            raise "Couldn't find event for %s" % content.absolute_url(relative=1)

        home = getToolByName(self, 'portal_membership').getHomeFolder()
        id = self.getUniqueID('smil_')
        home.invokeFactory(id=id, type_name='SmilObject')
        zLOG.LOG('portal_mediamanager', zLOG.INFO, '_createSmilObjectFor: id: %s' % (id))
        smil = getattr(home, id)
        zLOG.LOG('portal_mediamanager', zLOG.INFO, '_createSmilObjectFor: id: %s' % (smil.absolute_url()))
        smilurl = "%s/%s" % (home.absolute_url(relative=1), smil.getId())
        event.setSmil(smilurl) # we dont have to set event.setSmil
                                                    # as event.getSmil* takes the first
                                                    # if it is not defined.

        return smil

InitializeClass( MediaManagerTool )
