##############################################################################
#    Copyright (C) 2001  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################
"""
    Portal folders organize content.
"""
ADD_FOLDERS_PERMISSION = 'Add portal folders'

#Globals
import Globals, re, base64, marshal, string
import zLOG
from Globals import DTMLFile
from OFS.DTMLMethod import DTMLMethod
from AccessControl import getSecurityManager, ClassSecurityInfo
from Acquisition import aq_base, aq_parent, aq_inner
from OFS.Folder import Folder
from BTree import BTree
from DocumentTemplate.DT_Util import html_quote
from urllib import quote
from DateTime import DateTime

#CMF
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.CMFCorePermissions import AddPortalFolders
from Products.CMFCore.CMFCorePermissions import AddPortalContent
from Products.CMFCore.CMFCorePermissions import ViewManagementScreens
from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent, ManageProperties
from Products.CMFCore.utils import getToolByName, UniqueObject
from Products.CMFCore.PortalFolder import PortalFolder
from Products.CMFCore.PortalFolder import manage_addPortalFolder
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl

from ServerLogic import ServerLogic
from LiveServerLogic import LiveServerLogic

try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO

# Factory type information -- makes PortalBTreeFolder objects play nicely
# with the Types Tool (portal_types)
factory_type_information =  ({
     'id': 'Medianet',
     'meta_type': 'Media Manager Medianet',
     'description': 'Use folders to put content in categories.',
     'product': 'MyMediaManager',
     'icon': 'folder_icon.gif',
     'factory': 'manage_addMedianet',
     'filter_content_types': 0,
     'immediate_view': 'medianet_admin_form',
     'actions':
            ({'name': 'View',
	      'id': 'view',
              'action': 'medianet_view',
              'permissions': ('View',),
              'category': 'object'},
             {'name': 'Edit',
	      'id': 'edit',
              'action': 'medianet_admin_form',
              'permissions': ('Manage properties',),
              'category': 'object'},
             )
     },)

listtext0 = '''<select name="ids:list" multiple size="%s">
'''
listtext1 = '''<option>%s</option>
'''
listtext2 = '''</select>
'''

_marker = []  # Create a new marker object.

try:
    higher_getattr = PortalFolder.inheritedAttribute('__getattr__')
except:
    def higher_getattr(self, name):
        raise AttributeError, name

class Medianet( UniqueObject, PortalFolder, DefaultDublinCoreImpl ):
    """
        Like a PortalFolder but uses a BTree for storage with
        appropriate management UI.
    """
    id = 'medianet'
    meta_type = 'Media Manager Medianet'
    portal_type = 'Folder'

    manage_options=(
        ({'label':'Contents', 'action':'manage_main',},
         ) + Folder.manage_options[1:]
        )

    security = ClassSecurityInfo()

    description = ''


    def __init__( self, id, title='' ):
        DefaultDublinCoreImpl.__init__(self)
        self.id = id
        self.title = title
        self.live_logics = []
        self.ondemand_logics = []
        self.default_live = 0
        self.default_ondemand = 0
        self.default_slidelogic = 0

    security.declareProtected(View, 'SearchableText')
    def SearchableText(self):
        return "%s %s" % (self.Title(), self.Description())

    security.declareProtected(ModifyPortalContent, 'indexObject')
    def indexObject(self):
        catalog = getToolByName(self, 'portal_catalog', None)
        if catalog is not None:
            catalog.indexObject(self)

    security.declareProtected(ModifyPortalContent, 'unindexObject')
    def unindexObject(self):
        catalog = getToolByName(self, 'portal_catalog', None)
        if catalog is not None:
            catalog.unindexObject(self)

    security.declareProtected(ModifyPortalContent, 'reindexObject')
    def reindexObject(self):
        catalog = getToolByName(self, 'portal_catalog', None)
        if catalog is not None:
            catalog.reindexObject(self)

    security.declarePublic('getServerTypes')
    def getServerTypes(self):
        types = [{'id':'real',
            'title':'Real Media',
            'sequence_generator':'SMIL.smil'},
            {'id':'windows6',
            'title':'Windows Media 6.4',
            'sequence_generator':'SAMI.sam'},
            {'id':'windows7',
            'title':'Windows Media 7',
            'sequence_generator':'SAMI.sam'}]
        return types

    security.declarePublic('listLiveLogics')
    def listLiveLogics(self):
        logics = []
        ob = self

        while ob is not None and not getattr(ob, '_isPortalRoot', 0):
            ob = aq_parent(aq_inner(ob))

        i = 0
        while i < len(self.live_logics):
            logic = self.live_logics[i]
            try:
                if ob.restrictedTraverse(logic).meta_type == 'Media Manager Live Server Logic':
                    logics.append(logic)
                    i = i + 1
            except:
                del self.live_logics[i]
                self._p_changed = 1

        return tuple(logics)

    security.declarePublic('listLiveLogicObjects')
    def listLiveLogicObjects(self):
        logics = []
        ob = self
	while ob is not None and not getattr(ob, '_isPortalRoot', 0):
	    ob = aq_parent(aq_inner(ob))

	for logic in self.listLiveLogics():
	    try:
		logics.append(ob.restrictedTraverse(logic))
	    except:
		pass

	return tuple(logics)

    security.declarePublic('getLiveLogic')
    def getLiveLogic(self, lindex):
	index = int(lindex)
	return self.live_logics[index]

    security.declareProtected(ManageProperties, 'deleteLiveLogic')
    def deleteLiveLogic(self, lindex):
	index = int(lindex)
	logic = self.restrictedTraverse(self.live_logics[index])
	if index >= 0 and index < len(self.live_logics):
	   del self.live_logics[index]
	   self.manage_delObjects(logic.id)
	   self._p_changed = 1

    security.declareProtected(ManageProperties, 'addLiveLogic')
    def addLiveLogic(self, logic):
	for live_logic in self.live_logics:
	    if live_logic == logic:
		return

	self.live_logics.append(logic)
	self._p_changed = 1
	
    security.declareProtected(ManageProperties, 'setDefaultLive')
    def setDefaultLive(self, lindex):
	index = int(lindex)
	self.default_live = index

    security.declarePublic('getDefaultLive')
    def getDefaultLive(self):
	zLOG.LOG('Medianet', 0, 'Return default logic: %s' % (self.default_live))
	try:
	   logic = self.live_logics[int(self.default_live)]
	except:
	   raise 'Medianet error', 'No default live logic selected.\nPlease contact the system administrator'
	return logic

    security.declarePublic('listOndemandLogics')
    def listOndemandLogics(self):
	logics = []
	ob = self

	while ob is not None and not getattr(ob, '_isPortalRoot', 0):
	    ob = aq_parent(aq_inner(ob))

	i = 0
	while i < len(self.ondemand_logics):
	    logic = self.ondemand_logics[i]
	    try:
		if ob.restrictedTraverse(logic).meta_type == 'Media Manager Server Logic':
		   logics.append(logic)
		   i = i + 1
	    except:
		del self.ondemand_logics[i]
		self._p_changed = 1

	return tuple(logics)

    security.declarePublic('listLiveLogicObjects')
    def listOndemandLogicObjects(self):
        logics = []
	ob = self
	while ob is not None and not getattr(ob, '_isPortalRoot', 0):
	    ob = aq_parent(aq_inner(ob))

	for logic in self.listOndemandLogics():
	    try:
	       logics.append(ob.restrictedTraverse(logic))
	    except:
	       pass

        return tuple(logics)

    security.declarePublic('getOndemandLogic')
    def getOndemandLogic(self, lindex):
	index = int(lindex)
	return self.ondemand_logics[index]

    security.declareProtected(ManageProperties, 'deleteOndemandLogic')
    def deleteOndemandLogic(self, lindex):
	index = int(lindex)
	logic = self.restrictedTraverse(self.ondemand_logics[index])
	if index >= 0 and index < len(self.ondemand_logics):
	   del self.ondemand_logics[index]
	   self.manage_delObjects(logic.id)
	   self._p_changed = 1

    security.declareProtected(ManageProperties, 'addOndemandLogic')
    def addOndemandLogic(self, logic):
	for ondemand_logic in self.ondemand_logics:
	    if ondemand_logic == logic:
		return

	self.ondemand_logics.append(logic)
	self._p_changed = 1

    security.declareProtected(ManageProperties, 'setDefaultOndemand')
    def setDefaultOndemand(self, lindex):
        index = int(lindex)
        self.default_ondemand = index

    security.declarePublic('getDefaultOndemand')
    def getDefaultOndemand(self):
        try:
            logic = self.ondemand_logics[self.default_ondemand]
        except:
            raise 'Medianet error', 'No default live logic selected.\nPlease contact the system administrator'
        return logic

    security.declareProtected(ManageProperties, 'addLogic')
    def addLogic(self, title, type, servertype):
        if type == 'Live Server':
            uname = str(getSecurityManager().getUser())
            timestamp = str(DateTime().strftime('%d%m%y%H%M%S'))
            logic_id = uname + timestamp
            self.invokeFactory(id = logic_id, type_name=type)
            logic = self._getOb(logic_id)
            logic.edit(title=title, servertype=servertype)
        elif type == 'OnDemand Server':
            uname = str(getSecurityManager().getUser())
            timestamp = str(DateTime().strftime('%d%m%y%H%M%S'))
            logic_id = uname + timestamp
            self.invokeFactory(id = logic_id, type_name = type)
            logic = self._getOb(logic_id)
            logic.edit(title=title, servertype=servertype)
        else:
            raise 'Server type is not correct', 'You have set the servertype to %s. This type is not allowed' % (servertype)

    security.declareProtected(ManageProperties, 'setDefaultSlidelogic')
    def setDefaultSlidelogic(self, lindex):
        index = int(lindex)
        self.default_slidelogic = index

    security.declarePublic('getDefaultSlidelogic')
    def getDefaultSlidelogic(self):
        try:
            return self.ondemand_logics[self.default_slidelogic]
        except:
            if len(self.ondemand_logics) > 0:
                return self.ondemand_logics[0]

    security.declareProtected(ManageProperties, 'edit')
    def edit(self, title):
        if title is not None:
            self.setTitle(title)

    security.declareProtected(AddPortalFolders, 'manage_addMedianetFolder')
    def manage_addMedianet(self, id, title='', REQUEST=None):
        """Add a new MedianetFolder object with id *id*.
        """
        ob = Medianet(id, title)
        self._setObject(id, ob)
        if REQUEST is not None:
            return self.folder_contents(
                self, REQUEST, portal_status_message="Folder added")

    security.setPermissionDefault(AddPortalContent, ('Owner','Manager'))
    security.setPermissionDefault(AddPortalFolders, ('Owner','Manager'))


manage_addMedianet = Medianet.manage_addMedianet
manage_addMedianetForm = DTMLFile( 'folderAdd', globals() )

Globals.InitializeClass(Medianet)

# PortalContent classes need to be registered.  This installs the
# class's factory method and icon.
from Products.CMFCore.register import registerPortalContent
registerPortalContent(Medianet,
                      constructors=(Medianet.manage_addMedianet,),
                      action='manage_addMedianetForm',
                      icon="folder_icon.gif",
                      productGlobals=globals())
